import json

from .oauth import OAuth2Test


class MineIDOAuth2Test(OAuth2Test):
    backend_path = "social_core.backends.mineid.MineIDOAuth2"
    user_data_url = "https://www.mineid.org/api/user"
    expected_username = "foo@bar.com"
    access_token_body = json.dumps({"access_token": "foobar", "token_type": "bearer"})
    user_data_body = json.dumps(
        {
            "email": "foo@bar.com",
            "primary_profile": None,
        }
    )

    def test_login(self):
        self.do_login()

    def test_partial_pipeline(self):
        self.do_partial_pipeline()



from ..exceptions import InvalidEmail
from .partial import partial


@partial
def mail_validation(backend, details, is_new=False, *args, **kwargs):
    requires_validation = backend.REQUIRES_EMAIL_VALIDATION or backend.setting(
        "FORCE_EMAIL_VALIDATION", False
    )
    send_validation = details.get("email") and (
        is_new or backend.setting("PASSWORDLESS", False)
    )
    if requires_validation and send_validation:
        data = backend.strategy.request_data()
        if "verification_code" in data:
            backend.strategy.session_pop("email_validation_address")
            if not backend.strategy.validate_email(
                details["email"], data["verification_code"]
            ):
                raise InvalidEmail(backend)
        else:
            current_partial = kwargs.get("current_partial")
            backend.strategy.send_email_validation(
                backend, details["email"], current_partial.token
            )
            backend.strategy.session_set("email_validation_address", details["email"])
            return backend.strategy.redirect(
                backend.strategy.setting("EMAIL_VALIDATION_URL")
            )



"""
Auth0 implementation based on:
https://auth0.com/docs/quickstart/webapp/django/01-login
"""
from jose import jwt

from .oauth import BaseOAuth2


class Auth0OAuth2(BaseOAuth2):
    """Auth0 OAuth authentication backend"""

    name = "auth0"
    SCOPE_SEPARATOR = " "
    ACCESS_TOKEN_METHOD = "POST"
    EXTRA_DATA = [("picture", "picture")]

    def api_path(self, path=""):
        """Build API path for Auth0 domain"""
        return "https://{domain}/{path}".format(
            domain=self.setting("DOMAIN"), path=path
        )

    def authorization_url(self):
        return self.api_path("authorize")

    def access_token_url(self):
        return self.api_path("oauth/token")

    def get_user_id(self, details, response):
        """Return current user id."""
        return details["user_id"]

    def get_user_details(self, response):
        # Obtain JWT and the keys to validate the signature
        id_token = response.get("id_token")
        jwks = self.get_json(self.api_path(".well-known/jwks.json"))
        issuer = self.api_path()
        audience = self.setting("KEY")  # CLIENT_ID
        payload = jwt.decode(
            id_token, jwks, algorithms=["RS256"], audience=audience, issuer=issuer
        )
        fullname, first_name, last_name = self.get_user_names(payload["name"])
        return {
            "username": payload["nickname"],
            "email": payload["email"],
            "email_verified": payload.get("email_verified", False),
            "fullname": fullname,
            "first_name": first_name,
            "last_name": last_name,
            "picture": payload["picture"],
            "user_id": payload["sub"],
        }



import json
from urllib.parse import urlencode

from httpretty import HTTPretty

from ...actions import do_disconnect
from ..models import User
from .oauth import OAuth1Test, OAuth2Test
from .test_open_id_connect import OpenIdConnectTestMixin


class GoogleOAuth2Test(OAuth2Test):
    backend_path = "social_core.backends.google.GoogleOAuth2"
    user_data_url = "https://www.googleapis.com/oauth2/v3/userinfo"
    expected_username = "foo"
    access_token_body = json.dumps({"access_token": "foobar", "token_type": "bearer"})
    user_data_body = json.dumps(
        {
            "profile": "https://plus.google.com/101010101010101010101",
            "family_name": "Bar",
            "sub": "101010101010101010101",
            "picture": "https://lh5.googleusercontent.com/-ui-GqpNh5Ms/"
            "AAAAAAAAAAI/AAAAAAAAAZw/a7puhHMO_fg/photo.jpg",
            "locale": "en",
            "email_verified": True,
            "given_name": "Foo",
            "email": "foo@bar.com",
            "name": "Foo Bar",
        }
    )

    def test_login(self):
        self.do_login()
        last_request = HTTPretty.last_request
        self.assertEqual(last_request.method, "GET")
        self.assertTrue(self.user_data_url.endswith(last_request.path))
        self.assertEqual(
            last_request.headers["Authorization"],
            "Bearer foobar",
        )
        self.assertEqual(last_request.querystring, {})

    def test_partial_pipeline(self):
        self.do_partial_pipeline()

    def test_with_unique_user_id(self):
        self.strategy.set_settings(
            {
                "SOCIAL_AUTH_GOOGLE_OAUTH2_USE_UNIQUE_USER_ID": True,
            }
        )
        self.do_login()


class GoogleOAuth1Test(OAuth1Test):
    backend_path = "social_core.backends.google.GoogleOAuth"
    user_data_url = "https://www.googleapis.com/userinfo/email"
    expected_username = "foobar"
    access_token_body = json.dumps({"access_token": "foobar", "token_type": "bearer"})
    request_token_body = urlencode(
        {
            "oauth_token_secret": "foobar-secret",
            "oauth_token": "foobar",
            "oauth_callback_confirmed": "true",
        }
    )
    user_data_body = urlencode(
        {
            "email": "foobar@gmail.com",
            "isVerified": "true",
            "id": "101010101010101010101",
        }
    )

    def test_login(self):
        self.do_login()

    def test_partial_pipeline(self):
        self.do_partial_pipeline()

    def test_with_unique_user_id(self):
        self.strategy.set_settings(
            {"SOCIAL_AUTH_GOOGLE_OAUTH_USE_UNIQUE_USER_ID": True}
        )
        self.do_login()

    def test_with_anonymous_key_and_secret(self):
        self.strategy.set_settings(
            {
                "SOCIAL_AUTH_GOOGLE_OAUTH_KEY": None,
                "SOCIAL_AUTH_GOOGLE_OAUTH_SECRET": None,
            }
        )
        self.do_login()


class GoogleRevokeTokenTest(GoogleOAuth2Test):
    def test_revoke_token(self):
        self.strategy.set_settings(
            {"SOCIAL_AUTH_GOOGLE_OAUTH2_REVOKE_TOKENS_ON_DISCONNECT": True}
        )
        self.do_login()
        user = User.get(self.expected_username)
        user.password = "password"
        HTTPretty.register_uri(
            self._method(self.backend.REVOKE_TOKEN_METHOD),
            self.backend.REVOKE_TOKEN_URL,
            status=200,
        )
        do_disconnect(self.backend, user)


class GoogleOpenIdConnectTest(OpenIdConnectTestMixin, OAuth2Test):
    backend_path = "social_core.backends.google_openidconnect.GoogleOpenIdConnect"
    user_data_url = "https://www.googleapis.com/plus/v1/people/me/openIdConnect"
    issuer = "accounts.google.com"
    openid_config_body = json.dumps(
        {
            "issuer": "https://accounts.google.com",
            "authorization_endpoint": "https://accounts.google.com/o/oauth2/v2/auth",
            "token_endpoint": "https://www.googleapis.com/oauth2/v4/token",
            "userinfo_endpoint": "https://www.googleapis.com/oauth2/v3/userinfo",
            "revocation_endpoint": "https://accounts.google.com/o/oauth2/revoke",
            "jwks_uri": "https://www.googleapis.com/oauth2/v3/certs",
            "response_types_supported": [
                "code",
                "token",
                "id_token",
                "code token",
                "code id_token",
                "token id_token",
                "code token id_token",
                "none",
            ],
            "subject_types_supported": [
                "public",
            ],
            "id_token_signing_alg_values_supported": [
                "RS256",
            ],
            "scopes_supported": [
                "openid",
                "email",
                "profile",
            ],
            "token_endpoint_auth_methods_supported": [
                "client_secret_post",
                "client_secret_basic",
            ],
            "claims_supported": [
                "aud",
                "email",
                "email_verified",
                "exp",
                "family_name",
                "given_name",
                "iat",
                "iss",
                "locale",
                "name",
                "picture",
                "sub",
            ],
        }
    )



"""
Discord Auth OAuth2 backend, docs at:
    https://discord.com/developers/docs/topics/oauth2
"""
from .oauth import BaseOAuth2


class DiscordOAuth2(BaseOAuth2):
    name = "discord"
    HOSTNAME = "discord.com"
    AUTHORIZATION_URL = "https://%s/api/oauth2/authorize" % HOSTNAME
    ACCESS_TOKEN_URL = "https://%s/api/oauth2/token" % HOSTNAME
    ACCESS_TOKEN_METHOD = "POST"
    REVOKE_TOKEN_URL = "https://%s/api/oauth2/token/revoke" % HOSTNAME
    REVOKE_TOKEN_METHOD = "GET"
    DEFAULT_SCOPE = ["identify"]
    SCOPE_SEPARATOR = "+"
    REDIRECT_STATE = False
    EXTRA_DATA = [("expires_in", "expires"), ("refresh_token", "refresh_token")]

    def get_user_details(self, response):
        return {
            "username": response.get("username"),
            "email": response.get("email") or "",
        }

    def user_data(self, access_token, *args, **kwargs):
        url = "https://%s/api/users/@me" % self.HOSTNAME
        auth_header = {"Authorization": "Bearer %s" % access_token}
        return self.get_json(url, headers=auth_header)



from ...utils import PARTIAL_TOKEN_SESSION_NAME
from ..models import User
from .actions import BaseActionTest


class LoginActionTest(BaseActionTest):
    def test_login(self):
        self.do_login()

    def test_login_with_partial_pipeline(self):
        self.do_login_with_partial_pipeline()

    def test_fields_stored_in_session(self):
        self.strategy.set_settings(
            {"SOCIAL_AUTH_FIELDS_STORED_IN_SESSION": ["foo", "bar"]}
        )
        self.strategy.set_request_data({"foo": "1", "bar": "2"}, self.backend)
        self.do_login()
        self.assertEqual(self.strategy.session_get("foo"), "1")
        self.assertEqual(self.strategy.session_get("bar"), "2")

        self._logout(self.backend)
        # The _logout helper function doesn't clear the session.
        self.assertEqual(self.strategy.session_get("foo"), "1")
        self.assertEqual(self.strategy.session_get("bar"), "2")

        # Login again - without the 'bar' request param and make
        # sure its value didn't persist in the session.
        self.strategy.remove_from_request_data("bar")
        self.strategy.set_request_data({"foo": "3"}, self.backend)
        self.do_login()
        self.assertEqual(self.strategy.session_get("foo"), "3")
        self.assertEqual(self.strategy.session_get("bar"), None)

    def test_redirect_value(self):
        self.strategy.set_request_data({"next": "/after-login"}, self.backend)
        redirect = self.do_login(after_complete_checks=False)
        self.assertEqual(redirect.url, "/after-login")

    def test_login_with_invalid_partial_pipeline(self):
        def before_complete():
            partial_token = self.strategy.session_get(PARTIAL_TOKEN_SESSION_NAME)
            partial = self.strategy.storage.partial.load(partial_token)
            partial.data["backend"] = "foobar"

        self.do_login_with_partial_pipeline(before_complete)

    def test_new_user(self):
        self.strategy.set_settings({"SOCIAL_AUTH_NEW_USER_REDIRECT_URL": "/new-user"})
        redirect = self.do_login(after_complete_checks=False)
        self.assertEqual(redirect.url, "/new-user")

    def test_inactive_user(self):
        self.strategy.set_settings({"SOCIAL_AUTH_INACTIVE_USER_URL": "/inactive"})
        User.set_active(False)
        redirect = self.do_login(after_complete_checks=False)
        self.assertEqual(redirect.url, "/inactive")

    def test_invalid_user(self):
        self.strategy.set_settings(
            {
                "SOCIAL_AUTH_LOGIN_ERROR_URL": "/error",
                "SOCIAL_AUTH_PIPELINE": (
                    "social_core.pipeline.social_auth.social_details",
                    "social_core.pipeline.social_auth.social_uid",
                    "social_core.pipeline.social_auth.auth_allowed",
                    "social_core.pipeline.social_auth.social_user",
                    "social_core.pipeline.user.get_username",
                    "social_core.pipeline.user.create_user",
                    "social_core.pipeline.social_auth.associate_user",
                    "social_core.pipeline.social_auth.load_extra_data",
                    "social_core.pipeline.user.user_details",
                    "social_core.tests.pipeline.remove_user",
                ),
            }
        )
        redirect = self.do_login(after_complete_checks=False)
        self.assertEqual(redirect.url, "/error")



import json

from .oauth import OAuth2Test


class BaseLinkedinTest:
    user_data_url = (
        "https://api.linkedin.com/v2/me" "?projection=(firstName,id,lastName)"
    )
    expected_username = "FooBar"
    access_token_body = json.dumps({"access_token": "foobar", "token_type": "bearer"})

    # Reference:
    # https://docs.microsoft.com/en-us/linkedin/consumer/integrations/self
    # -serve/sign-in-with-linkedin?context=linkedin/consumer/context#api-request
    user_data_body = json.dumps(
        {
            "id": "1010101010",
            "firstName": {
                "localized": {"en_US": "Foo"},
                "preferredLocale": {"country": "US", "language": "en"},
            },
            "lastName": {
                "localized": {"en_US": "Bar"},
                "preferredLocale": {"country": "US", "language": "en"},
            },
        }
    )

    def test_login(self):
        self.do_login()

    def test_partial_pipeline(self):
        self.do_partial_pipeline()


class LinkedinOAuth2Test(BaseLinkedinTest, OAuth2Test):
    backend_path = "social_core.backends.linkedin.LinkedinOAuth2"


class LinkedinMobileOAuth2Test(BaseLinkedinTest, OAuth2Test):
    backend_path = "social_core.backends.linkedin.LinkedinMobileOAuth2"



import re
from os.path import dirname, join

from setuptools import setup

VERSION_RE = re.compile(r"__version__ = \"([\d\.]+)\"")

LONG_DESCRIPTION = """
Python Social Auth is an easy to setup social authentication/registration
mechanism with support for several frameworks and auth providers.

It implements a common interface to define new authentication
providers from third parties. And to bring support for more frameworks
and ORMs.
"""


def long_description():
    try:
        return open(join(dirname(__file__), "README.md")).read()
    except OSError:
        return None


def read_version():
    with open("social_core/__init__.py") as file:
        version_line = [
            line for line in file.readlines() if line.startswith("__version__")
        ][0]
        return VERSION_RE.match(version_line).groups()[0]


def read_requirements(filename):
    with open(filename) as file:
        return [line for line in file.readlines() if not line.startswith("-")]


def read_tests_requirements(filename):
    return read_requirements(f"social_core/tests/{filename}")


requirements = read_requirements("requirements-base.txt")
requirements_openidconnect = read_requirements("requirements-openidconnect.txt")
requirements_saml = read_requirements("requirements-saml.txt")
requirements_azuread = read_requirements("requirements-azuread.txt")

tests_requirements = read_tests_requirements("requirements.txt")

requirements_all = requirements_openidconnect + requirements_saml + requirements_azuread

tests_requirements = tests_requirements + requirements_all

setup(
    name="social-auth-core",
    version=read_version(),
    author="Matias Aguirre",
    author_email="matiasaguirre@gmail.com",
    description="Python social authentication made simple.",
    license="BSD",
    keywords="openid, oauth, saml, social auth",
    url="https://github.com/python-social-auth/social-core",
    packages=[
        "social_core",
        "social_core.backends",
        "social_core.pipeline",
        "social_core.tests",
        "social_core.tests.actions",
        "social_core.tests.backends",
        "social_core.tests.backends.data",
    ],
    long_description=long_description() or LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    install_requires=requirements,
    python_requires=">=3.6",
    extras_require={
        "openidconnect": [requirements_openidconnect],
        "saml": [requirements_saml],
        "azuread": [requirements_azuread],
        "all": [requirements_all],
        # Kept for compatibility
        "allpy3": [requirements_all],
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Internet",
        "License :: OSI Approved :: BSD License",
        "Intended Audience :: Developers",
        "Environment :: Web Environment",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    package_data={
        "social_core/tests": [
            "social_core/tests/*.txt",
            "social_core/tests/testkey.pem",
        ]
    },
    include_package_data=True,
    tests_require=tests_requirements,
    test_suite="social_core.tests",
    zip_safe=False,
)
